@@ -94,6 +94,11 @@ typedef __u32 __be32;
 #endif
 #endif /* < 2.6.9 */
 
+/* other missing types */
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 28)
+typedef unsigned int fmode_t;
+#endif
+
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
 /* for compat layer */
 #include <linux/pci.h>
@@ -962,6 +967,7 @@ void *snd_hidden_kmalloc(size_t size, gfp_t flags);
 void *snd_hidden_kzalloc(size_t size, gfp_t flags);
 void *snd_hidden_kcalloc(size_t n, size_t size, gfp_t flags);
 char *snd_hidden_kstrdup(const char *s, gfp_t flags);
+char *snd_hidden_kstrndup(const char *s, size_t len, gfp_t flags);
 void snd_hidden_kfree(const void *obj);
 
 static inline void *snd_wrapper_kmalloc(size_t size, gfp_t flags)
@@ -977,6 +983,7 @@ static inline void snd_wrapper_kfree(const void *obj)
 #define kzalloc(size, flags) snd_hidden_kzalloc(size, flags)
 #define kcalloc(n, size, flags) snd_hidden_kcalloc(n, size, flags)
 #define kstrdup(s, flags)  snd_hidden_kstrdup(s, flags)
+#define kstrndup(s, len, flags)  snd_hidden_kstrndup(s, len, flags)
 #define kfree(obj) snd_hidden_kfree(obj)
 
 #define kmalloc_nocheck(size, flags) snd_wrapper_kmalloc(size, flags)
@@ -997,6 +1004,11 @@ char *snd_compat_kstrdup(const char *s, gfp_t gfp_flags);
 #define kstrdup(s,f) snd_compat_kstrdup(s,f)
 #endif
 
+#ifndef CONFIG_HAVE_KSTRNDUP
+char *snd_compat_kstrndup(const char *s, size_t len, gfp_t gfp_flags);
+#define kstrndup(s,l,f) snd_compat_kstrndup(s,l,f)
+#endif
+
 #ifndef CONFIG_HAVE_KZALLOC
 void *snd_compat_kzalloc(size_t n, gfp_t gfp_flags);
 #define kzalloc(s,f) snd_compat_kzalloc(s,f)
@@ -1552,4 +1564,137 @@ static inline void put_unaligned_be64(u64 val, void *p)
 #define page_to_pfn(page)       (page_to_phys(page) >> PAGE_SHIFT)
 #endif
 
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 25)
+/* lazy wrapper - always returns 0 */
+static inline int __strict_strtoul(const char *cp, unsigned int base,
+				 unsigned long *valp)
+{
+	*valp = simple_strtoul(cp, NULL, base);
+	return 0;
+}
+#define strict_strtoul __strict_strtoul
+#endif /* < 2.6.25 */
+
+/* pr_xxx() macros */
+#ifndef pr_emerg
+#define pr_emerg(fmt, arg...) \
+	printk(KERN_EMERG fmt, ##arg)
+#endif
+#ifndef pr_alert
+#define pr_alert(fmt, arg...) \
+	printk(KERN_ALERT fmt, ##arg)
+#endif
+#ifndef pr_crit
+#define pr_crit(fmt, arg...) \
+	printk(KERN_CRIT fmt, ##arg)
+#endif
+#ifndef pr_err
+#define pr_err(fmt, arg...) \
+	printk(KERN_ERR fmt, ##arg)
+#endif
+#ifndef pr_warning
+#define pr_warning(fmt, arg...) \
+	printk(KERN_WARNING fmt, ##arg)
+#endif
+#ifndef pr_notice
+#define pr_notice(fmt, arg...) \
+	printk(KERN_NOTICE fmt, ##arg)
+#endif
+
+#ifndef WARN
+#define WARN(condition, arg...) ({ \
+	int __ret_warn_on = !!(condition);				\
+	if (unlikely(__ret_warn_on)) {					\
+		printk("WARNING: at %s:%d %s()\n",			\
+			__FILE__, __LINE__, __func__);			\
+		printk(arg);						\
+		dump_stack();						\
+	}								\
+	unlikely(__ret_warn_on);					\
+})
+#endif
+
+/* force to redefine WARN_ON() */
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 19)
+#undef WARN_ON
+#endif
+
+#ifndef WARN_ON
+#define WARN_ON(condition) ({						\
+	int __ret_warn_on = !!(condition);				\
+	if (unlikely(__ret_warn_on)) {					\
+		printk("WARNING: at %s:%d %s()\n",			\
+			__FILE__, __LINE__, __func__);			\
+		dump_stack();						\
+	}								\
+	unlikely(__ret_warn_on);					\
+})
+#endif
+
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 0)
+#define dev_printk(level, dev, format, arg...)	\
+	printk(level format, ##arg)
+#define dev_emerg(dev, format, arg...)		\
+	dev_printk(KERN_EMERG , dev , format , ## arg)
+#define dev_alert(dev, format, arg...)		\
+	dev_printk(KERN_ALERT , dev , format , ## arg)
+#define dev_crit(dev, format, arg...)		\
+	dev_printk(KERN_CRIT , dev , format , ## arg)
+#define dev_err(dev, format, arg...)		\
+	dev_printk(KERN_ERR , dev , format , ## arg)
+#define dev_warn(dev, format, arg...)		\
+	dev_printk(KERN_WARNING , dev , format , ## arg)
+#define dev_notice(dev, format, arg...)		\
+	dev_printk(KERN_NOTICE , dev , format , ## arg)
+#define dev_info(dev, format, arg...)		\
+	dev_printk(KERN_INFO , dev , format , ## arg)
+#ifdef DEBUG
+#define dev_dbg(dev, format, arg...)		\
+	dev_printk(KERN_DEBUG , dev , format , ## arg)
+#else
+#define dev_dbg(dev, format, arg...)		\
+	({ if (0) dev_printk(KERN_DEBUG, dev, format, ##arg); 0; })
+#endif
+#endif /* < 2.6.0 */
+
+/*
+ * wrapper for older SPARC codes with SBUS/EBUS specific bus
+ */
+#if defined(CONFIG_SBUS) && LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 28)
+struct sbus_dev;
+#define snd_dma_sbus_data(sbus)        ((struct device *)(sbus))
+#define SNDRV_DMA_TYPE_SBUS            4       /* SBUS continuous */
+#endif
+
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 22)
+#define DUMP_PREFIX_NONE	0
+#define DUMP_PREFIX_OFFSET	1
+void snd_compat_print_hex_dump_bytes(const char *prefix_str, int prefix_type,
+				     const void *buf, size_t len);
+#define print_hex_dump_bytes(a,b,c,d) snd_compat_print_hex_dump_bytes(a,b,c,d)
+#endif
+
+/*
+ * pci_ioremap_bar() wrapper
+ */
+#ifdef CONFIG_PCI
+#ifndef CONFIG_HAVE_PCI_IOREMAP_BAR
+#include <linux/pci.h>
+static inline void *pci_ioremap_bar(struct pci_dev *pdev, int bar)
+{
+	return ioremap_nocache(pci_resource_start(pdev, bar),
+			       pci_resource_len(pdev, bar));
+}
+#endif
+#endif
+
+/* pci_name() wrapper */
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 4, 23)
+#ifdef CONFIG_PCI
+#undef pci_name
+#define pci_name(pci)	((pci)->slot_name)
+#endif
+#endif
+
+
 #endif /* __SOUND_LOCAL_DRIVER_H */
