@@ -6,6 +6,7 @@
 #include <linux/list.h>
 #include <linux/pagemap.h>
 #include <linux/ioport.h>
+#include <asm/byteorder.h>
 
 #ifndef likely
 #if __GNUC__ == 2 && __GNUC_MINOR__ < 96
@@ -61,6 +62,18 @@ static __inline__ void list_del_init(struct list_head *entry)
 	INIT_LIST_HEAD(entry); 
 }
 
+/**
+ * list_move_tail - delete from one list and add as another's tail
+ * @list: the entry to move
+ * @head: the head that will follow our entry
+ */
+static inline void list_move_tail(struct list_head *list,
+				  struct list_head *head)
+{
+	__list_del(list->prev, list->next);
+	list_add_tail(list, head);
+}
+
 /**
  * list_for_each_safe	-	iterate over a list safe against removal of list entry
  * @pos:	the &struct list_head to use as a loop counter.
@@ -79,9 +92,22 @@ static __inline__ void list_del_init(struct list_head *entry)
  */
 #define list_for_each_entry(pos, head, member)				\
 	for (pos = list_entry((head)->next, typeof(*pos), member);	\
-	     pos->member.next, &pos->member != (head); 	\
+	     &pos->member != (head);					\
 	     pos = list_entry(pos->member.next, typeof(*pos), member))
 
+/**
+ * list_for_each_entry_safe - iterate over list of given type safe against removal of list entry
+ * @pos:	the type * to use as a loop counter.
+ * @n:		another type * to use as temporary storage
+ * @head:	the head for your list.
+ * @member:	the name of the list_struct within the struct.
+ */
+#define list_for_each_entry_safe(pos, n, head, member)			\
+	for (pos = list_entry((head)->next, typeof(*pos), member),	\
+		n = list_entry(pos->member.next, typeof(*pos), member);	\
+	     &pos->member != (head);					\
+	     pos = n, n = list_entry(n->member.next, typeof(*n), member))
+
 /* rw_semaphore - replaced with mutex */
 #define rw_semaphore semaphore
 #define init_rwsem(x) init_MUTEX(x)
@@ -105,6 +131,7 @@ static __inline__ void list_del_init(struct list_head *entry)
 	do { __save_flags(flags); __cli(); } while (0)
 #define local_irq_restore(flags) \
 	do { __restore_flags(flags); } while (0)
+#define local_irq_enable() __sti()
 
 /* Some distributions use modified kill_fasync */
 #ifdef CONFIG_OLD_KILL_FASYNC
@@ -144,6 +171,10 @@ struct tasklet_struct {
 
 #define rwlock_init(x) do { *(x) = RW_LOCK_UNLOCKED; } while(0)
 
+#ifndef fastcall
+#define fastcall
+#endif
+
 #ifndef __init
 #define __init
 #endif
@@ -219,6 +250,7 @@ struct resource {
 
 #define IORESOURCE_IO           0x00000100      /* Resource type */
 #define IORESOURCE_MEM		0x00000200
+#define IORESOURCE_CACHEABLE	0x00004000
 #endif
 
 static inline void snd_wrapper_request_region(unsigned long from, unsigned long extent, const char *name)
@@ -340,8 +372,8 @@ int snd_pci_compat_request_region(struct pci_dev *pdev, int bar, char *res_name)
 void snd_pci_compat_release_region(struct pci_dev *pdev, int bar);
 int snd_pci_compat_request_regions(struct pci_dev *pdev, char *res_name);
 void snd_pci_compat_release_regions(struct pci_dev *pdev);
-void snd_pci_compat_save_state(struct pci_dev *pdev);
-void snd_pci_compat_restore_state(struct pci_dev *pdev);
+int snd_pci_compat_save_state(struct pci_dev *pdev);
+int snd_pci_compat_restore_state(struct pci_dev *pdev);
 
 #define pci_module_init	snd_pci_compat_register_driver
 
@@ -602,4 +634,17 @@ static inline int abs(int val)
 
 #define might_sleep() do { } while (0)
 
+#ifndef __constant_cpu_to_le32
+#ifdef __LITTLE_ENDIAN
+#define __constant_cpu_to_le32(x) (x)
+#else
+#define __constant_cpu_to_le32(x) \
+	((__u32)( \
+		(((__u32)(x) & (__u32)0x000000ffUL) << 24) | \
+		(((__u32)(x) & (__u32)0x0000ff00UL) <<  8) | \
+		(((__u32)(x) & (__u32)0x00ff0000UL) >>  8) | \
+		(((__u32)(x) & (__u32)0xff000000UL) >> 24) ))
+#endif
+#endif
+
 #endif /* <2.3.0 */
